require 'fileutils'
require 'csv'
require 'date'

require 'perforat/results_processor'

module Perforat
  class Listener
    def start(_)
      current_time = Time.now.to_i
      FileUtils.mkdir_p(PERFORAT_DIR)
      @output = CSV.open("#{PERFORAT_DIR}/#{current_time}.csv", 'w')
      @output << [current_time]
    end

    def example_finished(notification)
      @output << [
        notification.example.full_description,
        notification.example.execution_result.run_time
      ]
    end

    def stop(_)
      @output.close
      puts
      files = Dir["#{PERFORAT_DIR}/*.csv"].sort_by(&File.method(:mtime))
      remove_old_files(files)
      contents = files.last(2).map { |file| CSV.read(file) }
      ResultsProcessor.new(contents: contents).call
    end

    alias :example_passed :example_finished
    alias :example_failed :example_finished

    private

    def remove_old_files(files)
      keep_files = Perforat.settings.fetch(:keep_files)
      (files - files.last(keep_files)).each(&File.method(:delete))
    end
  end
end
