module Perforat
  class ResultsProcessor
    def initialize(contents:, formatter: default_formatter, settings: Perforat.settings)
      @formatter = formatter
      @contents = contents
      @settings = settings
    end

    def call
      results = @contents.map(&parse_info).inject(&:merge)

      results.values.map(&:keys).inject(&:&).map do |test_name|
        durations = results.map(&tests_by(test_name)).reverse
        [test_name, durations, durations.inject(&:/)]
      end
      .sort_by(&by_coefficient_increasing)
      .last(settings.fetch(:results_count))
      .reject(&by_low_results)
      .each(&formatter)
    end

    private

    def parse_info
      lambda do |info|
        { parse_timestamp(info.shift.first) => info.map(&present_test_info).inject(&:merge) }
      end
    end

    def present_test_info
      -> (test, time) { { test => time.to_f } }
    end

    def tests_by(test_name)
      -> (_, tests) { tests.fetch(test_name) }
    end

    def by_coefficient_increasing
      -> (_, _, coefficient) { coefficient }
    end

    def by_low_results
      lambda do |_, durations, coefficient|
        durations.all? { |duration| duration < settings.fetch(:minimal_duration) } ||
          coefficient - 1 < settings.fetch(:minimal_epsilon)
      end
    end

    def parse_timestamp(timestamp)
      Time.at(timestamp.to_i).to_datetime
    end

    def default_formatter
      lambda do |(test_name, durations, coefficient)|
        puts "#{test_name}: #{durations[1]} -> #{durations[0]} coefficient: #{coefficient}"
      end
    end

    attr_reader :formatter, :settings
  end
end